<?php

namespace Litepie\Course\Providers;

use Illuminate\Support\ServiceProvider;

class CourseServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'course');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'course');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerCourse();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerCourse()
    {
        $this->app->bind('course', function($app) {
            return new Course($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Course', 'Lavalite\Course\Facades\Course');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('litepie.course', function ($app) {
            return $this->app->make('Litepie\Course\Course');
        });

        // Bind Course to repository
        $this->app->bind(
            'Litepie\Course\Interfaces\CourseRepositoryInterface',
            \Litepie\Course\Repositories\Eloquent\CourseRepository::class
        );        // Bind Subject to repository
        $this->app->bind(
            'Litepie\Course\Interfaces\SubjectRepositoryInterface',
            \Litepie\Course\Repositories\Eloquent\SubjectRepository::class
        );        // Bind Topic to repository
        $this->app->bind(
            'Litepie\Course\Interfaces\TopicRepositoryInterface',
            \Litepie\Course\Repositories\Eloquent\TopicRepository::class
        );

        $this->app->register(\Litepie\Course\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Litepie\Course\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and course's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/config.php', 'litepie.course'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeCourse::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['litepie.course'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('litepie/course.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/course')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/course')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
